/*
Copyright (c) Facebook, Inc. and its affiliates.
All rights reserved.

This source code is licensed under the BSD-style license found in the
LICENSE file in the root directory of this source tree.
*/

package main

import (
	"log"
	"time"

	"magma/feg/cloud/go/feg"
	"magma/feg/cloud/go/protos"
	"magma/feg/cloud/go/services/health"
	"magma/feg/cloud/go/services/health/reporter"
	"magma/feg/cloud/go/services/health/servicers"
	"magma/orc8r/cloud/go/blobstore"
	"magma/orc8r/cloud/go/service"
	"magma/orc8r/cloud/go/sqorc"
	"magma/orc8r/cloud/go/storage"

	"github.com/golang/glog"
)

const (
	NETWORK_HEALTH_STATUS_REPORT_INTERVAL = time.Second * 60
)

func main() {
	// Create the service
	srv, err := service.NewOrchestratorService(feg.ModuleName, health.ServiceName)
	if err != nil {
		log.Fatalf("Error creating service: %s", err)
	}
	db, err := sqorc.Open(storage.SQLDriver, storage.DatabaseSource)
	if err != nil {
		glog.Fatalf("Failed to connect to database: %s", err)
	}
	store := blobstore.NewEntStorage(health.DBTableName, db, sqorc.GetSqlBuilder())
	err = store.InitializeFactory()
	if err != nil {
		glog.Fatalf("Error initializing health database: %s", err)
	}
	// Add servicers to the service
	healthServer, err := servicers.NewHealthServer(store)
	protos.RegisterHealthServer(srv.GrpcServer, healthServer)

	// create a networkHealthStatusReporter to monitor and periodically log metrics
	// on if all gateways in a network are unhealthy
	healthStatusReporter := &reporter.NetworkHealthStatusReporter{}
	go healthStatusReporter.ReportHealthStatus(NETWORK_HEALTH_STATUS_REPORT_INTERVAL)

	// Run the service
	err = srv.Run()
	if err != nil {
		log.Fatalf("Error running service: %s", err)
	}
}
