// +build ignore
/*
Copyright (c) Facebook, Inc. and its affiliates.
All rights reserved.

This source code is licensed under the BSD-style license found in the
LICENSE file in the root directory of this source tree.
*/

package main

import (
	"flag"
	"fmt"
	"io"
	"os"

	"fbc/lib/go/radius/dictionary"
)

func main() {
	outputFile := flag.String("o", "-", "Output filename")
	flag.Parse()

	parser := &dictionary.Parser{
		Opener: &dictionary.FileSystemOpener{},
	}

	dict := &dictionary.Dictionary{}

	for _, filename := range flag.Args() {
		nextDict, err := parser.ParseFile(filename)
		if err != nil {
			fmt.Fprintln(os.Stderr, err)
			os.Exit(1)
		}

		dict, err = dictionary.Merge(dict, nextDict)
		if err != nil {
			fmt.Fprintln(os.Stderr, err)
			os.Exit(1)
		}
	}

	var w io.Writer
	if *outputFile == "-" {
		w = os.Stdout
	} else {
		f, err := os.OpenFile(*outputFile, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0644)
		if err != nil {
			fmt.Fprintln(os.Stderr, err)
			os.Exit(1)
		}
		defer f.Close()
		w = f
	}

	fmt.Fprintln(w, "// Code generated by generate_main.go. DO NOT EDIT.")
	fmt.Fprintln(w, "")
	fmt.Fprintln(w, "package debug")
	fmt.Fprintln(w, "")
	fmt.Fprintln(w, `import "fbc/lib/go/radius/dictionary"`)
	fmt.Fprintln(w, "")
	fmt.Fprintf(w, "var IncludedDictionary = %#v\n", dict)
}
