/*
 * Copyright (c) 2015, EURECOM (www.eurecom.fr)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the FreeBSD Project.
 */
/*! \file mme_config.h
  \brief
  \author Lionel Gauthier
  \company Eurecom
  \email: lionel.gauthier@eurecom.fr
*/

#ifndef FILE_MME_CONFIG_SEEN
#define FILE_MME_CONFIG_SEEN

#include <pthread.h>
#include <stdint.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include "mme_default_values.h"
#include "common_types.h"
#include "3gpp_23.003.h"
#include "3gpp_24.008.h"
#include "log.h"
#include "service303.h"

/* Currently supporting max 5 GUMMEI's in the mme configuration */
#define MIN_GUMMEI 1
#define MAX_GUMMEI 5
#define MIN_TAI_SUPPORTED 1
#define MAX_TAI_SUPPORTED 16
#define MAX_MCC_LENGTH 3
#define MAX_MNC_LENGTH 3
#define MIN_MNC_LENGTH 2
#define CIDR_SPLIT_LIST_COUNT 2

#define MME_CONFIG_STRING_MME_CONFIG "MME"
#define MME_CONFIG_STRING_PID_DIRECTORY "PID_DIRECTORY"
#define MME_CONFIG_STRING_RUN_MODE "RUN_MODE"
#define MME_CONFIG_STRING_RUN_MODE_TEST "TEST"
#define MME_CONFIG_STRING_REALM "REALM"
#define MME_CONFIG_STRING_MAXENB "MAXENB"
#define MME_CONFIG_STRING_MAXUE "MAXUE"
#define MME_CONFIG_STRING_RELATIVE_CAPACITY "RELATIVE_CAPACITY"
#define MME_CONFIG_STRING_STATISTIC_TIMER "MME_STATISTIC_TIMER"

#define MME_CONFIG_STRING_IP_CAPABILITY "IP_CAPABILITY"
#define MME_CONFIG_STRING_USE_STATELESS "USE_STATELESS"
#define MME_CONFIG_STRING_FULL_NETWORK_NAME "FULL_NETWORK_NAME"
#define MME_CONFIG_STRING_SHORT_NETWORK_NAME "SHORT_NETWORK_NAME"
#define MME_CONFIG_STRING_DAYLIGHT_SAVING_TIME "DAYLIGHT_SAVING_TIME"
#define MME_CONFIG_STRING_CSFB_CONFIG "CSFB"
#define MME_CONFIG_STRING_NON_EPS_SERVICE_CONTROL "NON_EPS_SERVICE_CONTROL"

#define MME_CONFIG_STRING_EMERGENCY_ATTACH_SUPPORTED                           \
  "EMERGENCY_ATTACH_SUPPORTED"
#define MME_CONFIG_STRING_UNAUTHENTICATED_IMSI_SUPPORTED                       \
  "UNAUTHENTICATED_IMSI_SUPPORTED"

#define EPS_NETWORK_FEATURE_SUPPORT_IMS_VOICE_OVER_PS_SESSION_IN_S1            \
  "EPS_NETWORK_FEATURE_SUPPORT_IMS_VOICE_OVER_PS_SESSION_IN_S1"
#define EPS_NETWORK_FEATURE_SUPPORT_EMERGENCY_BEARER_SERVICES_IN_S1_MODE       \
  "EPS_NETWORK_FEATURE_SUPPORT_EMERGENCY_BEARER_SERVICES_IN_S1_MODE"
#define EPS_NETWORK_FEATURE_SUPPORT_LOCATION_SERVICES_VIA_EPC                  \
  "EPS_NETWORK_FEATURE_SUPPORT_LOCATION_SERVICES_VIA_EPC"
#define EPS_NETWORK_FEATURE_SUPPORT_EXTENDED_SERVICE_REQUEST                   \
  "EPS_NETWORK_FEATURE_SUPPORT_EXTENDED_SERVICE_REQUEST"

#define MME_CONFIG_STRING_INTERTASK_INTERFACE_CONFIG "INTERTASK_INTERFACE"
#define MME_CONFIG_STRING_INTERTASK_INTERFACE_QUEUE_SIZE "ITTI_QUEUE_SIZE"

#define MME_CONFIG_STRING_S6A_CONFIG "S6A"
#define MME_CONFIG_STRING_S6A_CONF_FILE_PATH "S6A_CONF"
#define MME_CONFIG_STRING_S6A_HSS_HOSTNAME "HSS_HOSTNAME"

#define MME_CONFIG_STRING_SCTP_CONFIG "SCTP"
#define MME_CONFIG_STRING_SCTP_INSTREAMS "SCTP_INSTREAMS"
#define MME_CONFIG_STRING_SCTP_OUTSTREAMS "SCTP_OUTSTREAMS"

#define MME_CONFIG_STRING_S1AP_CONFIG "S1AP"
#define MME_CONFIG_STRING_S1AP_OUTCOME_TIMER "S1AP_OUTCOME_TIMER"
#define MME_CONFIG_STRING_S1AP_PORT "S1AP_PORT"

#define MME_CONFIG_STRING_GUMMEI_LIST "GUMMEI_LIST"
#define MME_CONFIG_STRING_MME_CODE "MME_CODE"
#define MME_CONFIG_STRING_MME_GID "MME_GID"
#define MME_CONFIG_STRING_TAI_LIST "TAI_LIST"
#define MME_CONFIG_STRING_MCC "MCC"
#define MME_CONFIG_STRING_MNC "MNC"
#define MME_CONFIG_STRING_TAC "TAC"

#define MME_CONFIG_STRING_NETWORK_INTERFACES_CONFIG "NETWORK_INTERFACES"
#define MME_CONFIG_STRING_INTERFACE_NAME_FOR_S1_MME                            \
  "MME_INTERFACE_NAME_FOR_S1_MME"
#define MME_CONFIG_STRING_IPV4_ADDRESS_FOR_S1_MME "MME_IPV4_ADDRESS_FOR_S1_MME"
#define MME_CONFIG_STRING_INTERFACE_NAME_FOR_S11_MME                           \
  "MME_INTERFACE_NAME_FOR_S11_MME"
#define MME_CONFIG_STRING_IPV4_ADDRESS_FOR_S11_MME                             \
  "MME_IPV4_ADDRESS_FOR_S11_MME"
#define MME_CONFIG_STRING_MME_PORT_FOR_S11 "MME_PORT_FOR_S11_MME"

#define MME_CONFIG_STRING_NAS_CONFIG "NAS"
#define MME_CONFIG_STRING_NAS_SUPPORTED_INTEGRITY_ALGORITHM_LIST               \
  "ORDERED_SUPPORTED_INTEGRITY_ALGORITHM_LIST"
#define MME_CONFIG_STRING_NAS_SUPPORTED_CIPHERING_ALGORITHM_LIST               \
  "ORDERED_SUPPORTED_CIPHERING_ALGORITHM_LIST"

#define MME_CONFIG_STRING_NAS_T3402_TIMER "T3402"
#define MME_CONFIG_STRING_NAS_T3412_TIMER "T3412"
#define MME_CONFIG_STRING_NAS_T3422_TIMER "T3422"
#define MME_CONFIG_STRING_NAS_T3450_TIMER "T3450"
#define MME_CONFIG_STRING_NAS_T3460_TIMER "T3460"
#define MME_CONFIG_STRING_NAS_T3470_TIMER "T3470"
#define MME_CONFIG_STRING_NAS_T3485_TIMER "T3485"
#define MME_CONFIG_STRING_NAS_T3486_TIMER "T3486"
#define MME_CONFIG_STRING_NAS_T3489_TIMER "T3489"
#define MME_CONFIG_STRING_NAS_T3495_TIMER "T3495"
#define MME_CONFIG_STRING_NAS_FORCE_REJECT_TAU "FORCE_REJECT_TAU"
#define MME_CONFIG_STRING_NAS_FORCE_REJECT_SR "FORCE_REJECT_SR"
#define MME_CONFIG_STRING_NAS_DISABLE_ESM_INFORMATION_PROCEDURE                \
  "DISABLE_ESM_INFORMATION_PROCEDURE"
#define MME_CONFIG_STRING_NAS_FORCE_PUSH_DEDICATED_BEARER                      \
  "FORCE_PUSH_DEDICATED_BEARER"

#define MME_CONFIG_STRING_SGS_CONFIG "SGS"
#define MME_CONFIG_STRING_SGS_TS6_1_TIMER "TS6_1"
#define MME_CONFIG_STRING_SGS_TS8_TIMER "TS8"
#define MME_CONFIG_STRING_SGS_TS9_TIMER "TS9"
#define MME_CONFIG_STRING_SGS_TS10_TIMER "TS10"
#define MME_CONFIG_STRING_SGS_TS13_TIMER "TS13"

#define MME_CONFIG_STRING_ASN1_VERBOSITY "ASN1_VERBOSITY"
#define MME_CONFIG_STRING_ASN1_VERBOSITY_NONE "none"
#define MME_CONFIG_STRING_ASN1_VERBOSITY_ANNOYING "annoying"
#define MME_CONFIG_STRING_ASN1_VERBOSITY_INFO "info"
#define MME_CONFIG_STRING_SGW_LIST_SELECTION "S-GW_LIST_SELECTION"
#define MME_CONFIG_STRING_ID "ID"

#define MAGMA_CONFIG_STRING "MAGMA"
#define MME_CONFIG_STRING_SERVICE303_CONFIG "SERVICE303"
#define MME_CONFIG_STRING_SERVICE303_CONF_SERVER_ADDRESS "SERVER_ADDRESS"
//CSFB
#define MME_CONFIG_STRING_CSFB_MCC "CSFB_MCC"
#define MME_CONFIG_STRING_CSFB_MNC "CSFB_MNC"
#define MME_CONFIG_STRING_LAC "LAC"

typedef enum { RUN_MODE_TEST = 0, RUN_MODE_OTHER } run_mode_t;

typedef struct eps_network_feature_config_s {
  uint8_t ims_voice_over_ps_session_in_s1;
  uint8_t emergency_bearer_services_in_s1_mode;
  uint8_t location_services_via_epc;
  uint8_t extended_service_request;
} eps_network_feature_config_t;

#define TRACKING_AREA_IDENTITY_LIST_TYPE_ONE_PLMN_NON_CONSECUTIVE_TACS 0x00
#define TRACKING_AREA_IDENTITY_LIST_TYPE_ONE_PLMN_CONSECUTIVE_TACS 0x01
#define TRACKING_AREA_IDENTITY_LIST_TYPE_MANY_PLMNS 0x02
typedef struct served_tai_s {
  uint8_t list_type;
  uint8_t nb_tai;
  uint16_t *plmn_mcc;
  uint16_t *plmn_mnc;
  uint16_t *plmn_mnc_len;
  uint16_t *tac;
} served_tai_t;

typedef struct sctp_config_s {
  uint16_t in_streams;
  uint16_t out_streams;
} sctp_config_t;

typedef struct s1ap_config_s {
  uint16_t port_number;
  uint8_t outcome_drop_timer_sec;
} s1ap_config_t;

typedef struct ipv4_s {
  bstring if_name_s1_mme;
  struct in_addr s1_mme;
  int netmask_s1_mme;

  bstring if_name_s11;
  struct in_addr s11;
  int netmask_s11;
  uint16_t port_s11;
} ipv4_t;

typedef struct s6a_config_s {
  bstring conf_file;
  bstring hss_host_name;
} s6a_config_t;

typedef struct itti_config_s {
  uint32_t queue_size;
  bstring log_file;
} itti_config_t;

typedef struct nas_config_s {
  uint8_t prefered_integrity_algorithm[8];
  uint8_t prefered_ciphering_algorithm[8];
  uint32_t t3402_min;
  uint32_t t3412_min;
  uint32_t t3422_sec;
  uint32_t t3450_sec;
  uint32_t t3460_sec;
  uint32_t t3470_sec;
  uint32_t t3485_sec;
  uint32_t t3486_sec;
  uint32_t t3489_sec;
  uint32_t t3495_sec;
  // non standard features
  bool force_reject_tau;
  bool force_reject_sr;
  bool disable_esm_information;
} nas_config_t;

typedef struct sgs_config_s {
  uint32_t ts6_1_sec;
  uint32_t ts8_sec;
  uint32_t ts9_sec;
  uint32_t ts10_sec;
  uint32_t ts13_sec;
} sgs_config_t;

#define MME_CONFIG_MAX_SGW 16
typedef struct e_dns_config_s {
  int nb_sgw_entries;
  bstring sgw_id[MME_CONFIG_MAX_SGW];
  struct in_addr sgw_ip_addr[MME_CONFIG_MAX_SGW];
} e_dns_config_t;

typedef struct gummei_config_s {
  int nb;
  gummei_t gummei[MAX_GUMMEI];
} gummei_config_t;

typedef struct mme_config_s {
  /* Reader/writer lock for this configuration */
  pthread_rwlock_t rw_lock;

  bstring config_file;
  bstring pid_dir;
  bstring realm;
  bstring full_network_name;
  bstring short_network_name;
  uint8_t daylight_saving_time;

  run_mode_t run_mode;

  uint32_t max_enbs;
  uint32_t max_ues;

  uint8_t relative_capacity;

  uint32_t mme_statistic_timer;

  bstring ip_capability;
  bstring non_eps_service_control;

  uint8_t unauthenticated_imsi_supported;

  eps_network_feature_config_t eps_network_feature_support;

  gummei_config_t gummei;

  served_tai_t served_tai;

  service303_data_t service303_config;
  sctp_config_t sctp_config;
  s1ap_config_t s1ap_config;
  s6a_config_t s6a_config;
  itti_config_t itti_config;
  nas_config_t nas_config;
  sgs_config_t sgs_config;
  log_config_t log_config;
  e_dns_config_t e_dns_emulation;

  ipv4_t ipv4;

  lai_t lai;

  bool use_stateless;
} mme_config_t;

extern mme_config_t mme_config;

int mme_config_find_mnc_length(
  const char mcc_digit1P,
  const char mcc_digit2P,
  const char mcc_digit3P,
  const char mnc_digit1P,
  const char mnc_digit2P,
  const char mnc_digit3P);

void mme_config_init(mme_config_t *);
int mme_config_parse_opt_line(int argc, char *argv[], mme_config_t *mme_config);
int mme_config_parse_file(mme_config_t *);
void mme_config_display(mme_config_t *);

void mme_config_exit(void);

#define mme_config_read_lock(mMEcONFIG)                                        \
  pthread_rwlock_rdlock(&(mMEcONFIG)->rw_lock)
#define mme_config_write_lock(mMEcONFIG)                                       \
  pthread_rwlock_wrlock(&(mMEcONFIG)->rw_lock)
#define mme_config_unlock(mMEcONFIG)                                           \
  pthread_rwlock_unlock(&(mMEcONFIG)->rw_lock)

#endif /* FILE_MME_CONFIG_SEEN */
