#  Copyright (c) Facebook, Inc. and its affiliates.
#  All rights reserved.
#
#  This source code is licensed under the BSD-style license found in the
#  LICENSE file in the root directory of this source tree.
import unittest
from time import sleep

import s1ap_types
from integ_tests.s1aptests import s1ap_wrapper


class TestAttachDetachMobilitydRestart(unittest.TestCase):

    def setUp(self):
        self._s1ap_wrapper = s1ap_wrapper.TestWrapper()

    def tearDown(self):
        self._s1ap_wrapper.cleanup()

    def test_attach_detach_mobility_restart(self):
        """ Basic attach/detach test with a single UE and mobilityd restart """
        num_ues = 2
        detach_type = [s1ap_types.ueDetachType_t.UE_NORMAL_DETACH.value,
                       s1ap_types.ueDetachType_t.UE_SWITCHOFF_DETACH.value]
        wait_for_s1 = [True, False]
        self._s1ap_wrapper.configUEDevice(num_ues)

        for i in range(num_ues):
            req = self._s1ap_wrapper.ue_req
            print("************************* Running End to End attach for ",
                  "UE id ", req.ue_id)

            # Now actually complete the attach
            self._s1ap_wrapper._s1_util.attach(
                req.ue_id, s1ap_types.tfwCmd.UE_END_TO_END_ATTACH_REQUEST,
                s1ap_types.tfwCmd.UE_ATTACH_ACCEPT_IND,
                s1ap_types.ueAttachAccept_t)

            # Wait on EMM Information from MME
            self._s1ap_wrapper._s1_util.receive_emm_info()

            print('************************* Restarting mobilityd')
            self._s1ap_wrapper.magmad_util.restart_services(['mobilityd'])
            # Timeout for mobilityd restart
            for j in range(30):
                print("Waiting for", j, "seconds")
                sleep(1)

            print("************************* Running UE detach for UE id ",
                  req.ue_id)
            # Now detach the UE
            self._s1ap_wrapper.s1_util.detach(
                req.ue_id, detach_type[i], wait_for_s1[i])


if __name__ == "__main__":
    unittest.main()
